﻿/****************** Copyright Notice *****************
 
This code is licensed under Microsoft Public License (Ms-PL). 
You are free to use, modify and distribute any portion of this code. 
The only requirement to do that, you need to keep the developer name, as provided below to recognize and encourage original work:

=======================================================
   
Architecture Designed and Implemented By:
Mohammad Ashraful Alam
Microsoft Most Valuable Professional, ASP.NET 2007 – 2011
Twitter: http://twitter.com/AshrafulAlam | Blog: http://blog.ashraful.net | Portfolio: http://www.ashraful.net
   
*******************************************************/

using System;
using System.Collections.Generic;
using System.Data.Services.Client;
using System.Linq;
using System.Linq.Dynamic;
using Eisk.DomainAccess.DataServiceClient.ServiceReference;

namespace Eisk.BusinessLogicLayer.Strategies.DataService
{
    [System.ComponentModel.DataObject(true)]
    public partial class EmployeeBLL : IDisposable
    {
        #region Constructors, Dependency and Partial Method Declaration

        public EmployeeBLL() : this(new DatabaseContext(new Uri("http://localhost:3800/Eisk-DataService.svc/"))) { }

        public EmployeeBLL(DatabaseContext DatabaseContext)
        {
            _DatabaseContext = DatabaseContext;
        }

        DatabaseContext _DatabaseContext;
        const int EmployeeIdMinValue = 0;

        public void Dispose()
        {
            //_DatabaseContext.Dispose();
        }

        partial void OnEmployeeSaving(Employee employee);

        partial void OnEmployeeCreating(Employee employee);
        partial void OnEmployeeCreated(Employee employee);

        partial void OnEmployeeUpdating(Employee employee);
        partial void OnEmployeeUpdated(Employee employee);

        partial void OnEmployeeSaved(Employee employee);

        partial void OnEmployeeDeleting(Employee employee);
        partial void OnEmployeeDeleted(Employee employee);


        #endregion

        #region Get Methods

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Select, true)]
        public Employee GetEmployeeByEmployeeId(int employeeId)
        {
            //Validate Input
            if (employeeId <= EmployeeIdMinValue)
                throw (new ArgumentOutOfRangeException("employeeId"));

            return (from employee in _DatabaseContext.Employees
                    where employee.EmployeeId == employeeId
             select employee).ToList()[0];
            
        }

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Select, false)]
        public List<Employee> GetEmployeesByReportsTo(int? reportsTo)
        {
            //Validate Input
            if (reportsTo == 0)
                return GetAllEmployees();
            else if (reportsTo < EmployeeIdMinValue)
                throw (new ArgumentOutOfRangeException("reportsTo"));

            List<Employee> employees;

            if (reportsTo == null)
                employees = (from employee in _DatabaseContext.Employees
                    where employee.ReportsTo == null
                    select employee).ToList();
            else
                employees = (from employee in _DatabaseContext.Employees
                    where employee.ReportsTo == reportsTo
                    select employee).ToList();

            return employees;
        }

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Select, false)]
        public List<Employee> GetEmployeesByReportsToPaged(int? reportsTo, string orderBy, int startRowIndex, int maximumRows)
        {
            //Validate Input
            if (reportsTo == 0)
                return GetAllEmployeesPaged(orderBy, startRowIndex, maximumRows);
            else if (reportsTo < EmployeeIdMinValue)
                throw (new ArgumentOutOfRangeException("reportsTo"));

            //If there is no sorting parameter, then by default consider the primary key as the sorting field
            if (string.IsNullOrEmpty(orderBy))
                orderBy = "EmployeeId";

            List<Employee> employees;

            if (reportsTo == null)
                employees = (
                    from employee in
                        _DatabaseContext.Employees
                    .DynamicOrderBy(orderBy)
                    where employee.ReportsTo == null
                    select employee
                    )
                    .Skip(startRowIndex)
                    .Take(maximumRows)
                    .ToList();
            else
                employees = (
                    from employee in
                        _DatabaseContext.Employees
                    .DynamicOrderBy(orderBy)
                    where employee.ReportsTo == reportsTo
                    select employee
                    )
                    .Skip(startRowIndex)
                    .Take(maximumRows)
                    .ToList();

            return employees;
                    
        }

        public int GetEmployeesByReportsToPagedCount(int? reportsTo, string orderBy, int startRowIndex, int maximumRows)
        {
            //Validate Input
            if (reportsTo == 0)
                return GetAllEmployeesPagedCount(orderBy, startRowIndex, maximumRows);
            else if (reportsTo < EmployeeIdMinValue)
                throw (new ArgumentOutOfRangeException("reportsTo"));
            
            return GetEmployeesByReportsTo(reportsTo).Count;

            //if (reportsTo == null)
            //    return _DatabaseContext.Employees.Count(employee => employee.ReportsTo == null);
            //else
            //    return _DatabaseContext.Employees.Count(employee => employee.ReportsTo == reportsTo);
               
        }

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Select, false)]
        public List<Employee> GetAllEmployees()
        {
            return _DatabaseContext.Employees.ToList();
        }

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Select, false)]
        public List<Employee> GetAllEmployeesPaged(string orderBy, int startRowIndex, int maximumRows)
        {
            if (string.IsNullOrEmpty(orderBy))
                orderBy = "EmployeeId";

            return (
                    from employee in 
                        _DatabaseContext.Employees
                        .DynamicOrderBy(orderBy)
                    select employee
                    )
                    .Skip(startRowIndex)
                    .Take(maximumRows)
                    .ToList();
        }

        public int GetAllEmployeesPagedCount(string orderBy, int startRowIndex, int maximumRows)
        {
            return _DatabaseContext.Employees.Count();
        }

        #endregion

        #region Persistence (Create, Update, Delete) Methods

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Insert, true)]
        public int CreateNewEmployee(Employee newEmployee)
        {
            // Validate Parameters
            if (newEmployee == null)
                throw (new ArgumentNullException("newEmployee"));

            // Validate Primary key value
            if (newEmployee.EmployeeId > EmployeeIdMinValue)
                throw (new ArgumentOutOfRangeException("newEmployee"));

            // Apply business rules
            OnEmployeeSaving(newEmployee);
            OnEmployeeCreating(newEmployee);

            // Perform parsistance operation
            _DatabaseContext.AddObject("Employees", newEmployee);
            _DatabaseContext.SaveChanges();

            //int numberOfAffectedRows = _DatabaseContext.SaveChanges();
            //if (numberOfAffectedRows == 0) throw new InvalidOperationException("No employee created!");

            // Apply business workflow
            OnEmployeeCreated(newEmployee);
            OnEmployeeSaved(newEmployee);

            return newEmployee.EmployeeId;
        }

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Update, true)]
        public void UpdateEmployee(Employee updatedEmployee)
        {
            // Validate Parameters
            if (updatedEmployee == null)
                throw (new ArgumentNullException("updatedEmployee"));

            // Validate Primary key value
            if (updatedEmployee.EmployeeId <= EmployeeIdMinValue)
                throw (new ArgumentOutOfRangeException("updatedEmployee"));

            // Apply business rules
            OnEmployeeSaving(updatedEmployee);
            OnEmployeeUpdating(updatedEmployee);

            //attaching and making ready for parsistance

            //_DatabaseContext.MergeOption = MergeOption.AppendOnly;
            _DatabaseContext.AttachTo("Employees", updatedEmployee);
            _DatabaseContext.UpdateObject(updatedEmployee);
            DataServiceResponse response = _DatabaseContext.SaveChanges();
            
            //int numberOfAffectedRows = 0;
            //if (numberOfAffectedRows == 0) throw new InvalidOperationException("No employee updated!");

            //Apply business workflow
            OnEmployeeUpdated(updatedEmployee);
            OnEmployeeSaved(updatedEmployee);

        }

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Delete, true)]
        public void DeleteEmployee(Employee employeeToBeDeleted)
        {
            //Validate Input
            if (employeeToBeDeleted == null)
                throw (new ArgumentNullException("employeeToBeDeleted"));

            // Validate Primary key value
            if (employeeToBeDeleted.EmployeeId <= EmployeeIdMinValue)
                throw (new ArgumentOutOfRangeException("employeeToBeDeleted"));

            OnEmployeeSaving(employeeToBeDeleted);
            OnEmployeeDeleting(employeeToBeDeleted);

            _DatabaseContext.AttachTo("Employees", employeeToBeDeleted);
            _DatabaseContext.DeleteObject(employeeToBeDeleted);
            DataServiceResponse response = _DatabaseContext.SaveChanges();

            //int numberOfAffectedRows = _DatabaseContext.SaveChanges();
            //if (numberOfAffectedRows == 0) throw new InvalidOperationException("No employee deleted!");
            
            OnEmployeeDeleted(employeeToBeDeleted);
            OnEmployeeSaved(employeeToBeDeleted);

        }

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Delete, false)]
        public void DeleteEmployee(int employeeId)
        {
            DeleteEmployee (new Employee {EmployeeId = employeeId});
            
        }

        public void DeleteEmployees(List<int> employeesIdsToDelete)
        {
            //Validate Input
            foreach (int employeeId in employeesIdsToDelete)
                if (employeeId <= EmployeeIdMinValue)
                    throw (new ArgumentOutOfRangeException("employeesIdsToDelete"));

            List<Employee> employeesToBeDeleted = new List<Employee>();

            foreach (int employeeId in employeesIdsToDelete)
            {
                Employee employeeToBeDeleted = new Employee { EmployeeId = employeeId };
                _DatabaseContext.AttachTo("Employees", employeeToBeDeleted);
                _DatabaseContext.DeleteObject(employeeToBeDeleted);
                employeesToBeDeleted.Add(employeeToBeDeleted);
                OnEmployeeDeleting(employeeToBeDeleted);

            }
            
            DataServiceResponse response = _DatabaseContext.SaveChanges();

            //int numberOfAffectedRows = _DatabaseContext.SaveChanges();
            //if (numberOfAffectedRows != employeesIdsToDelete.Count) throw new InvalidOperationException("One or more employee records have not been deleted.");

            foreach (Employee employeeToBeDeleted in employeesToBeDeleted)
                OnEmployeeDeleted(employeeToBeDeleted);
        }

        #endregion

    }//end of Employee class
}

