﻿/****************** Copyright Notice *****************
 
This code is licensed under Microsoft Public License (Ms-PL). 
You are free to use, modify and distribute any portion of this code. 
The only requirement to do that, you need to keep the developer name, as provided below to recognize and encourage original work:

=======================================================
   
Architecture Designed and Implemented By:
Mohammad Ashraful Alam
Microsoft Most Valuable Professional, ASP.NET 2007 – 2011
Twitter: http://twitter.com/AshrafulAlam | Blog: http://blog.ashraful.net | Portfolio: http://www.ashraful.net
   
*******************************************************/

using System;
using System.Collections.Generic;
using System.Data.Objects;
using System.Linq;
using Eisk.BusinessEntities;
using Eisk.DataAccessLayer;

namespace Eisk.BusinessLogicLayer.Strategies.CompiledQueryWrapper
{
    [System.ComponentModel.DataObject(true)]
    public partial class EmployeeBLL : GenericBusinessLogicComponentLinqToEntities<Employee>, IEmployeeBLL
    {
        public EmployeeBLL()
            : this(new DatabaseContext())
        {
        }

        public EmployeeBLL(DatabaseContext DatabaseContext)
            : base(DatabaseContext)
        {
        }

        #region Get Methods 

        // GetEmployeeByEmployeeId

        static readonly Func<DatabaseContext, int, Employee> compiledQuery_GetEmployeeByEmployeeId =
                    CompiledQuery.Compile<DatabaseContext, int, Employee>(
            (ctx, employeeId) => ctx.Employees.FirstOrDefault(empObj => empObj.EmployeeId == employeeId));


        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Select, true)]
        public Employee GetEmployeeByEmployeeId(int employeeId)
        {
            return compiledQuery_GetEmployeeByEmployeeId.Invoke(_DatabaseContext, employeeId);
        }

        // GetEmployeesByReportsTo

        static readonly Func<DatabaseContext, int?, IQueryable<Employee>> compiledQuery_GetEmployeesByReportsTo =
                   CompiledQuery.Compile<DatabaseContext, int?, IQueryable<Employee>>(
           (ctx, reportsTo) => (from employee in ctx.Employees
                                where employee.ReportsTo == reportsTo
                                select employee));

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Select, false)]
        public List<Employee> GetEmployeesByReportsTo(int? reportsTo)
        {
            return compiledQuery_GetEmployeesByReportsTo.Invoke (_DatabaseContext, reportsTo).ToList();
        }

        // GetEmployeesByReportsToPage

        static readonly Func<DatabaseContext, int?, string, int, int, IQueryable<Employee>> compiledQuery_GetEmployeesByReportsToPaged =
                   CompiledQuery.Compile<DatabaseContext, int?, string, int, int, IQueryable<Employee>>(
           (ctx, reportsTo, orderBy, startRowIndex, maximumRows) =>
               (
                    from employee in
                        ctx.Employees
                        orderby employee.EmployeeId
                        //.DynamicOrderBy(orderBy) //need sorting support!
                    where employee.ReportsTo == reportsTo
                    select employee
                    )
                    .Skip(startRowIndex)
                    .Take(maximumRows)
             );

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Select, false)]
        public List<Employee> GetEmployeesByReportsToPaged(int? reportsTo, string orderBy, int startRowIndex, int maximumRows)
        {
            return compiledQuery_GetEmployeesByReportsToPaged.Invoke (_DatabaseContext, reportsTo, orderBy, startRowIndex, maximumRows).ToList();
        }

        // GetEmployeesByReportsToPagedCount

        static readonly Func<DatabaseContext, int?, int> compiledQuery_GetTotalCountForAllEmployeesByReportsTo =
                   CompiledQuery.Compile<DatabaseContext, int?, int>(
            (
               ctx, reportsTo) =>
               ctx.Employees.Count(employee => employee.ReportsTo == reportsTo)
            );

        public int GetTotalCountForAllEmployeesByReportsTo(int? reportsTo, string orderBy, int startRowIndex, int maximumRows)
        {
            return compiledQuery_GetTotalCountForAllEmployeesByReportsTo.Invoke(_DatabaseContext, reportsTo);
        }

        // GetAllEmployees
        
        static readonly Func<DatabaseContext, IQueryable<Employee>> compiledQuery_GetAllEmployees =
                   CompiledQuery.Compile<DatabaseContext, IQueryable<Employee>>(
           (ctx) => (from employee in ctx.Employees
                                select employee));

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Select, false)]
        public List<Employee> GetAllEmployees()
        {
            return compiledQuery_GetAllEmployees.Invoke(_DatabaseContext).ToList();
        }

        // GetAllEmployeesPaged

        static readonly Func<DatabaseContext, string, int, int, IQueryable<Employee>> compiledQuery_GetAllEmployeesPaged =
                   CompiledQuery.Compile<DatabaseContext, string, int, int, IQueryable<Employee>>(
           (ctx, orderBy, startRowIndex, maximumRows) =>
               (
                    from employee in
                        ctx.Employees
                    orderby employee.EmployeeId
                    //.DynamicOrderBy(orderBy) // need sorting support!
                    select employee
                    )
                    .Skip(startRowIndex)
                    .Take(maximumRows)
             );

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Select, false)]
        public List<Employee> GetAllEmployeesPaged(string orderBy, int startRowIndex, int maximumRows)
        {
            return compiledQuery_GetAllEmployeesPaged.Invoke(_DatabaseContext, orderBy, startRowIndex, maximumRows).ToList();
        }

        // GetAllEmployeesPagedCount

        static readonly Func<DatabaseContext, int> compiledQuery_GetTotalCountForAllEmployees =
                   CompiledQuery.Compile<DatabaseContext, int>(
            (ctx) =>
               ctx.Employees.Count()
            );

        public int GetTotalCountForAllEmployees(string orderBy, int startRowIndex, int maximumRows)
        {
            return compiledQuery_GetTotalCountForAllEmployees.Invoke(_DatabaseContext);
        }

        #endregion

        #region Persistence (Create, Update, Delete) Methods

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Insert, true)]
        public int CreateNewEmployee(Employee newEmployee)
        {
            return (int)base.CreateNewEntity(newEmployee)[0].Value;
        }

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Update, true)]
        public void UpdateEmployee(Employee updatedEmployee)
        {
            base.UpdateEntity(updatedEmployee);
        }

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Delete, true)]
        public void DeleteEmployee(Employee employeeToBeDeleted)
        {
            base.DeleteEntity(employeeToBeDeleted);        
        }

        public void DeleteEmployees(List<int> employeesIdsToDelete)
        {
            base.DeleteEntities(emp => employeesIdsToDelete.Contains(emp.EmployeeId));
        }

        #endregion

    }
}

