﻿/****************** Copyright Notice *****************
 
This code is licensed under Microsoft Public License (Ms-PL). 
You are free to use, modify and distribute any portion of this code. 
The only requirement to do that, you need to keep the developer name, as provided below to recognize and encourage original work:

=======================================================
   
Architecture Designed and Implemented By:
Mohammad Ashraful Alam
Microsoft Most Valuable Professional, ASP.NET 2007 – 2011
Twitter: http://twitter.com/AshrafulAlam | Blog: http://blog.ashraful.net | Portfolio: http://www.ashraful.net
   
*******************************************************/

using System;
using System.Collections.Generic;
using System.Data;
using System.Data.Objects;
using System.Data.Objects.DataClasses;
using System.Linq;
using System.Linq.Dynamic;
using Eisk.DataAccessLayer;
namespace Eisk.BusinessLogicLayer.Strategies
{
    public partial class GenericBusinessLogicComponentLinqToEntities<Entity> : BusinessLogicComponentBase where Entity : EntityObject
    {
        IObjectSet<Entity> _entityObjectSet;

        public GenericBusinessLogicComponentLinqToEntities()
            : this(new DatabaseContext())
        {
        }

        public GenericBusinessLogicComponentLinqToEntities(DatabaseContext DatabaseContext)
            : base(DatabaseContext)
        {
            _entityObjectSet = _DatabaseContext.CreateObjectSet<Entity>();
        }

        #region  Data Operations

        protected Entity GetSingleEntityByCriteria(Func<Entity, bool> predicate)
        {
            return _entityObjectSet.SingleOrDefault(predicate);
        }

        protected List<Entity> GetEntitiesByCriteria(Func<Entity, bool> predicate)
        {
            return _entityObjectSet.Where(predicate).ToList();
        }

        protected List<Entity> GetEntitiesByCriteriaAsPaged(Func<Entity, bool> predicate, string orderBy, int startRowIndex, int maximumRows)
        {
            return (
                    from entity in
                        _entityObjectSet
                        .DynamicOrderBy(orderBy)
                        .Where(predicate)
                    select entity
                    )
                    .Skip(startRowIndex)
                    .Take(maximumRows)
                    .ToList();
        }

        protected int GetTotalCountForAllEntitiesByCriteria(Func<Entity, bool> predicate)
        {
            return _entityObjectSet.Count(predicate);
        }


        protected List<Entity> GetAllEntities()
        {
            return _entityObjectSet.ToList();
        }

        protected List<Entity> GetAllEntitiesAsPaged(string orderBy, int startRowIndex, int maximumRows)
        {
            return (
                    from entity in
                        _entityObjectSet
                        .DynamicOrderBy(orderBy)
                    select entity
                    )
                    .Skip(startRowIndex)
                    .Take(maximumRows)
                    .ToList();
        }

        protected int GetTotalCountForAllEntities()
        {
            return _entityObjectSet.Count();
        }

        protected EntityKeyMember[] CreateNewEntity(Entity newEntity)
        {
            AttachAndChangeState(newEntity, EntityState.Added);
            int numberOfAffectedRows = _DatabaseContext.SaveChanges();
            if (numberOfAffectedRows == 0)
                throw new InvalidOperationException("No entity created!");

            return newEntity.EntityKey.EntityKeyValues;
        }

        protected void UpdateEntity(Entity updatedEntity)
        {
            AttachAndChangeState(updatedEntity, EntityState.Modified);
            int numberOfAffectedRows = _DatabaseContext.SaveChanges();
            if (numberOfAffectedRows == 0)
                throw new InvalidOperationException("No entity updated!");
        }

        protected void DeleteEntity(Entity entityToBeDeleted)
        {
            AttachAndChangeState(entityToBeDeleted, EntityState.Deleted);
            int numberOfAffectedRows = _DatabaseContext.SaveChanges();
            if (numberOfAffectedRows == 0)
                throw new InvalidOperationException("No entity deleted!");
        }

        protected void DeleteEntities(Func<Entity, bool> predicate)
        {
            List<Entity> entities = (from entity in _entityObjectSet.Where<Entity>(predicate) select entity).ToList();

            foreach (Entity entity in entities)
            {
                AttachAndChangeState(entity, EntityState.Deleted);
            }

            var numberOfAffectedRows = _DatabaseContext.SaveChanges();
            if (numberOfAffectedRows != entities.Count)
                throw new InvalidOperationException("One or more entity records have not been deleted.");
        }

        void AttachAndChangeState(Entity entity, System.Data.EntityState entityState)
        {
            if (entity.EntityState == EntityState.Detached)
                _entityObjectSet.Attach(entity);
            _DatabaseContext.ObjectStateManager.ChangeObjectState(entity, entityState);
        }

        #endregion

    }
}

