﻿/****************** Copyright Notice *****************
 
This code is licensed under Microsoft Public License (Ms-PL). 
You are free to use, modify and distribute any portion of this code. 
The only requirement to do that, you need to keep the developer name, as provided below to recognize and encourage original work:

=======================================================
   
Architecture Designed and Implemented By:
Mohammad Ashraful Alam
Microsoft Most Valuable Professional, ASP.NET 2007 – 2011
Twitter: http://twitter.com/AshrafulAlam | Blog: http://blog.ashraful.net | Portfolio: http://www.ashraful.net
   
*******************************************************/

using System.Collections.Generic;
using Eisk.BusinessEntities;
using Eisk.BusinessLogicLayer.Strategies;
using Eisk.DataAccessLayer;

namespace Eisk.BusinessLogicLayer.Strategies.LinqToEntitiesWrapper
{
    [System.ComponentModel.DataObject(true)]
    public partial class EmployeeBLL : GenericBusinessLogicComponentLinqToEntities<Employee>, IEmployeeBLL
    {
        public EmployeeBLL():this(new DatabaseContext())
        {
        }

        public EmployeeBLL(DatabaseContext DatabaseContext)
            : base(DatabaseContext)
        {
        }

        #region Get Methods

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Select, true)]
        public Employee GetEmployeeByEmployeeId(int employeeId)
        {
            return base.GetSingleEntityByCriteria(empObj => empObj.EmployeeId == employeeId);
        }

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Select, false)]
        public List<Employee> GetEmployeesByReportsTo(int? reportsTo)
        {
            return base.GetEntitiesByCriteria(employee => reportsTo == null ? employee.ReportsTo == null : employee.ReportsTo == reportsTo);
        }

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Select, false)]
        public List<Employee> GetEmployeesByReportsToPaged(int? reportsTo, string orderBy, int startRowIndex, int maximumRows)
        {
            return base.GetEntitiesByCriteriaAsPaged(employee => reportsTo == null ? employee.ReportsTo == null : employee.ReportsTo == reportsTo, orderBy, startRowIndex, maximumRows);
        }

        public int GetTotalCountForAllEmployeesByReportsTo(int? reportsTo, string orderBy, int startRowIndex, int maximumRows)
        {
            return base.GetTotalCountForAllEntitiesByCriteria(employee => reportsTo == null ? employee.ReportsTo == null : employee.ReportsTo == reportsTo);
        }

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Select, false)]
        public List<Employee> GetAllEmployees()
        {
            return base.GetAllEntities();
        }

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Select, false)]
        public List<Employee> GetAllEmployeesPaged(string orderBy, int startRowIndex, int maximumRows)
        {
            return base.GetAllEntitiesAsPaged(orderBy, startRowIndex, maximumRows);
        }

        public int GetTotalCountForAllEmployees(string orderBy, int startRowIndex, int maximumRows)
        {
            return base.GetTotalCountForAllEntities();
        }

        #endregion

        #region Persistence (Create, Update, Delete) Methods

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Insert, true)]
        public int CreateNewEmployee(Employee newEmployee)
        {
            return (int)base.CreateNewEntity(newEmployee)[0].Value;
        }

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Update, true)]
        public void UpdateEmployee(Employee updatedEmployee)
        {
            base.UpdateEntity(updatedEmployee);
        }

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Delete, true)]
        public void DeleteEmployee(Employee employeeToBeDeleted)
        {
            base.DeleteEntity(employeeToBeDeleted);
        }

        public void DeleteEmployees(List<int> employeesIdsToDelete)
        {
           base.DeleteEntities (emp =>  employeesIdsToDelete.Contains (emp.EmployeeId));
        }

        #endregion

    }
}

