/****************** Copyright Notice *****************
 
This code is licensed under Microsoft Public License (Ms-PL). 
You are free to use, modify and distribute any portion of this code. 
The only requirement to do that, you need to keep the developer name, as provided below to recognize and encourage original work:

=======================================================
   
Architecture Designed and Implemented By:
Mohammad Ashraful Alam
Microsoft Most Valuable Professional, ASP.NET 2007  2011
Twitter: http://twitter.com/AshrafulAlam | Blog: http://blog.ashraful.net | Portfolio: http://www.ashraful.net
   
*******************************************************/

using System;
using System.Collections.Generic;
using System.Web;
using Eisk.BusinessEntities;
namespace Eisk.BusinessLogicLayer.Strategies.CacheProxy
{
    [System.ComponentModel.DataObject(true)]
    public sealed class EmployeeBLL:IEmployeeBLL
    {
        #region Constructors, Dependency

        IEmployeeBLL _employeeBLL;

        public EmployeeBLL()
        {
            string configProvider = System.Configuration.ConfigurationSettings.AppSettings["CacheLayerStrategy"];

            //if (string.IsNullOrEmpty(configProvider) || configProvider == "Eisk.BusinessLogicLayer")
            //    _employeeBLL = new Eisk.BusinessLogicLayer.EmployeeBLL();
            //else
                _employeeBLL = (IEmployeeBLL)Activator.CreateInstance(Type.GetType(configProvider + ".EmployeeBLL"));
        }

        public EmployeeBLL(IEmployeeBLL employeeBLL)
        {
            _employeeBLL = employeeBLL;
        }

       public void Dispose()
        {
            if (_employeeBLL != null)
            {
                _employeeBLL.Dispose();
                _employeeBLL = null;
            }

            GC.SuppressFinalize(this);
        }

        #endregion

        #region Cache Utility methods

        /// <summary>
        /// This is the key which enables us to reduce the chanse to overlap the chache key.
        /// In this case the master key is concated with individual cache items in the current entity.
        /// </summary>
        private static readonly string[] MasterCacheKeyArray = { "EmployeesCache" };
        
        /// <summary>
        /// Gets the chache key regadring the passed 'raw' key
        /// </summary>
        /// <param name="rawKey"></param>
        /// <returns></returns>
        private static string GetCacheKey(string rawKey)
        {
            return string.Concat(MasterCacheKeyArray[0], "-", rawKey);
        }

        /// <summary>
        /// Gets the cache item regarding the modified key of 'raw' key
        /// </summary>
        /// <param name="rawKey"></param>
        /// <returns></returns>
        private static object GetCacheItem(string rawKey)
        {
            return HttpRuntime.Cache[GetCacheKey(rawKey)];
        }

        /// <summary>
        /// Adds the item to the web cache, along with general dependency support
        /// </summary>
        /// <param name="rawKey"></param>
        /// <param name="value"></param>
        private static void AddCacheItem(string rawKey, object value)
        {
            System.Web.Caching.Cache DataCache = HttpRuntime.Cache;

            // Make sure MasterCacheKeyArray[0] is in the cache and create a depedency
            DataCache[MasterCacheKeyArray[0]] = DateTime.Now;
            System.Web.Caching.CacheDependency masterCacheKeyDependency = new System.Web.Caching.CacheDependency(null, MasterCacheKeyArray);

            //we are putting the cache without any expiration policy, 
            //we are expiring the cache programatically once when a new or existing instance of the current entity are being updated.
            DataCache.Insert(GetCacheKey(rawKey), value, masterCacheKeyDependency, System.Web.Caching.Cache.NoAbsoluteExpiration, System.Web.Caching.Cache.NoSlidingExpiration);
        }

        #region SQL Cache Dependency

        ///// <summary>
        ///// Adds the item to the web cache, along with sql dependency support and general dependency support
        ///// To support sql cache dependeny, add the following xml snippet in web.config file
        /////<system.web>
        /////    <!-- Configure the polling service used for SQL cache dependencies
        /////    To set-up sql cache dependency in db level lets use the following command:
        /////    directory: C:\WINDOWS\Microsoft.NET\Framework\v2.0.50727
        /////    db level:
        /////    aspnet_regsql.exe -S ASHRAF\SQLEXPRESS -E -d pubs -ed
        /////    table level:
        /////    aspnet_regsql.exe -S ASHRAF\SQLEXPRESS -E -d pubs -t authors -et
        /////    -->
        /////    <caching>
        /////      <sqlCacheDependency enabled="true" pollTime="1000" >
        /////        <databases>
        /////          <add name="SCDEmployeeDB" connectionStringName="EmployeeDB" />
        /////        </databases>
        /////      </sqlCacheDependency>
        /////    </caching>
        /////</system.web>
        ///// </summary>
        ///// <param name="rawKey"></param>
        ///// <param name="value"></param>
        ///// <param name="databaseEntryName">The name of a database defined in the databases element of the application's Web.config file. </param>
        ///// <param name="tableNames">The name of the database table that the System.Web.Caching.SqlCacheDependency is associated with. </param>
        //public static void AddCacheItem(string rawKey, object value, string databaseEntryName, string[] tableNames)
        //{
        //    System.Web.Caching.Cache DataCache = HttpRuntime.Cache;

        //    // Make sure MasterCacheKeyArray[0] is in the cache and create a depedency
        //    DataCache[MasterCacheKeyArray[0]] = DateTime.Now;
        //    System.Web.Caching.CacheDependency masterCacheKeyDependency = new System.Web.Caching.CacheDependency(null, MasterCacheKeyArray);

        //    //Add the SqlCacheDependency objects for relevant tables
        //    CacheDependency[] cacheDependencies = new CacheDependency[tableNames.Length + 1];

        //    cacheDependencies[0] = masterCacheKeyDependency;//adding the general dependency

        //    for (int i = 1; i < cacheDependencies.Length; i++)//adding sql dependencies
        //    {
        //        cacheDependencies[i] = new SqlCacheDependency(databaseEntryName, tableNames[i - 1]);
        //    }

        //    // Create an AggregateCacheDependency
        //    System.Web.Caching.AggregateCacheDependency aggregateDependencies = new System.Web.Caching.AggregateCacheDependency();
        //    aggregateDependencies.Add(cacheDependencies);

        //    DataCache.Insert(GetCacheKey(rawKey), value, aggregateDependencies, System.Web.Caching.Cache.NoAbsoluteExpiration, System.Web.Caching.Cache.NoSlidingExpiration);        
        //}

        #endregion

        /// <summary>
        /// This method is called proramatically by the insert/update methods to expire the cache item.
        /// All the cache items related to the current entity is bound with dependency relation, which causes all the cache items related to 
        /// the current entity to be expired after calling this method.
        /// </summary>
        private static void InvalidateCache()
        {
            // Remove the cache dependency
            HttpRuntime.Cache.Remove(MasterCacheKeyArray[0]);
        }

        #endregion

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Select, true)]
        public Employee GetEmployeeByEmployeeId(int employeeId)
        {
            string rawKey = string.Concat("EISK_GetEmployeeByEmployeeId" + "-", employeeId);
            
            // See if the item is in the cache
            Employee employee = GetCacheItem(rawKey) as Employee;
            if (employee == null)
            {
                // Item not found in cache - retrieve it and insert it into the cache
                employee = _employeeBLL.GetEmployeeByEmployeeId(employeeId);
                AddCacheItem(rawKey, employee);
            }

            return employee;
        }

        public List<Employee> GetEmployeesByReportsTo(int? reportsTo)
        {
            string rawKey = string.Concat("EISK_GetEmployeesByReportsTo" + "-", reportsTo.ToString());
            // See if the item is in the cache
            List<Employee> Employees = GetCacheItem(rawKey) as List<Employee>;
            if (Employees == null)
            {
                // Item not found in cache - retrieve it and insert it into the cache
                Employees = _employeeBLL.GetEmployeesByReportsTo(reportsTo);
                AddCacheItem(rawKey, Employees);
            }

            return Employees;
        }

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Select, false)]
        public List<Employee> GetEmployeesByReportsToPaged(int? reportsTo, string orderBy, int startRowIndex, int maximumRows)
        {
            string rawKey = string.Concat("EISK_GetEmployeesByReportsToPaged" + "-", reportsTo + "-" + orderBy + "-" + startRowIndex + "-" + maximumRows);
            // See if the item is in the cache
            List<Employee> Employees = GetCacheItem(rawKey) as List<Employee>;
            if (Employees == null)
            {
                // Item not found in cache - retrieve it and insert it into the cache
                Employees = _employeeBLL.GetEmployeesByReportsToPaged(reportsTo, orderBy, startRowIndex, maximumRows);
                AddCacheItem(rawKey, Employees);
            }

            return Employees;
        }

        public int GetTotalCountForAllEmployeesByReportsTo(int? reportsTo, string orderBy, int startRowIndex, int maximumRows)
        {
            string rawKey = string.Concat("EISK_GetTotalCountForAllEmployeesByReportsTo" + "-", reportsTo + "-" + orderBy + "-" + startRowIndex + "-" + maximumRows);
            // See if the item is in the cache
            int pagedCount = Convert.ToInt32(GetCacheItem(rawKey), System.Globalization.CultureInfo.CurrentCulture.NumberFormat);
            if (pagedCount == 0)
            {
                // Item not found in cache - retrieve it and insert it into the cache
                pagedCount = _employeeBLL.GetTotalCountForAllEmployeesByReportsTo(reportsTo, orderBy, startRowIndex, maximumRows);
                AddCacheItem(rawKey, pagedCount);
            }

            return pagedCount;
        }

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Select, false)]
        public List<Employee> GetAllEmployees()
        {
            const string rawKey = "EISK_GetAllEmployees";
            // See if the item is in the cache
            List<Employee> Employees = GetCacheItem(rawKey) as List<Employee>;
            if (Employees == null)
            {
                // Item not found in cache - retrieve it and insert it into the cache
                Employees = _employeeBLL.GetAllEmployees();
                AddCacheItem(rawKey, Employees);
            }

            return Employees;
        }

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Select, false)]
        public List<Employee> GetAllEmployeesPaged(String orderBy, int startRowIndex, int maximumRows)
        {
            string rawKey = string.Concat("EISK_GetAllEmployeesPaged" + "-", orderBy + "-" + startRowIndex + "-" + maximumRows);
            // See if the item is in the cache
            List<Employee> Employees = GetCacheItem(rawKey) as List<Employee>;
            if (Employees == null)
            {
                // Item not found in cache - retrieve it and insert it into the cache
                Employees = _employeeBLL.GetAllEmployeesPaged(orderBy, startRowIndex, maximumRows);
                AddCacheItem(rawKey, Employees);
            }

            return Employees;
        }

        public int GetTotalCountForAllEmployees(string orderBy, int startRowIndex, int maximumRows)
        {
            string rawKey = string.Concat("EISK_GetTotalCountForAllEmployees" + "-", orderBy + "-" + startRowIndex + "-" + maximumRows);
            // See if the item is in the cache
            int pagedCount = Convert.ToInt32(GetCacheItem(rawKey), System.Globalization.CultureInfo.CurrentCulture.NumberFormat);
            if (pagedCount == 0)
            {
                // Item not found in cache - retrieve it and insert it into the cache
                pagedCount = _employeeBLL.GetTotalCountForAllEmployees(orderBy, startRowIndex, maximumRows);
                AddCacheItem(rawKey, pagedCount);
            }

            return pagedCount;
        }

        
        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Insert, true)]
        public int CreateNewEmployee(Employee newEmployee)
        {
            int result = _employeeBLL.CreateNewEmployee(newEmployee);
            //Invalidate the chache
            InvalidateCache();
            return result;
        }

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Update, true)]
        public void UpdateEmployee(Employee updatedEmployee)
        {
            _employeeBLL.UpdateEmployee(updatedEmployee);
            //Invalidate the chache
            InvalidateCache();
        }

        [System.ComponentModel.DataObjectMethodAttribute(System.ComponentModel.DataObjectMethodType.Delete, true)]
        public void DeleteEmployee(Employee employeeToBeDeleted)
        {
            _employeeBLL.DeleteEmployee(employeeToBeDeleted);
            //Invalidate the chache
            InvalidateCache();
        }

        public void DeleteEmployees(List<int> employeesIdsToDelete)
        {
            _employeeBLL.DeleteEmployees(employeesIdsToDelete);
            //Invalidate the chache
            InvalidateCache();
        }
        
    }
}
