﻿using System;
using Eisk.Helpers;
namespace Eisk.BusinessEntities
{
    partial class Employee
    {
        const string nullValueErrorMessage = "Data field is not allowed to be null. Data field name: ";
        const string emptyValueErrorMessage = "Data field is not allowed to be empty. Data field name: ";

        //primary key, required field
        partial void OnEmployeeIdChanging(global::System.Int32 value)
        {
            if (value.IsInvalidKey()) throw new InvalidOperationException(emptyValueErrorMessage + "EmployeeId");
        }

        //required field
        partial void OnLastNameChanging(global::System.String value)
        {
            if (value.IsNull()) throw new InvalidOperationException(nullValueErrorMessage + "LastName");
            else if (value.IsEmpty()) throw new InvalidOperationException(emptyValueErrorMessage + "LastName");
        }

        //required field
        partial void OnFirstNameChanging(global::System.String value)
        {
            if (value.IsNull()) throw new InvalidOperationException(nullValueErrorMessage + "FirstName");
            else if (value.IsEmpty()) throw new InvalidOperationException(emptyValueErrorMessage + "FirstName");
        }

        /// <summary>
        /// Validation for optional field
        /// For Optional data fields, when no data is provied, those are stored in database as NULL.
        /// Thus no empty values are allowed for optional data fields.
        /// </summary>
        /// <param name="value"></param>
        partial void OnTitleChanging(global::System.String value)
        {
            if (value.IsEmpty()) throw new InvalidOperationException(emptyValueErrorMessage + "Title");
        }

        //optional field
        partial void OnTitleOfCourtesyChanging(global::System.String value)
        {
            if (value.IsEmpty()) throw new InvalidOperationException(emptyValueErrorMessage + "TitleOfCourtesy");
        }

        //optional field
        partial void OnBirthDateChanging(global::System.DateTime? value)
        {
            if (value.IsEmpty()) throw new InvalidOperationException(emptyValueErrorMessage + "BirthDate");
        }

        //optional field
        partial void OnBirthDateChanged()
        {
            ValidateBirthDateAndHireDate();
        }

        //required field
        partial void OnHireDateChanging(global::System.DateTime value)
        {
            if (value.IsEmpty()) throw new InvalidOperationException(emptyValueErrorMessage + "HireDate");
        }
        
        //required field
        partial void OnHireDateChanged()
        {
            ValidateBirthDateAndHireDate();
        }

        /// <summary>
        /// validation method
        /// RULE: BirthDate should not be later than the HireDate
        /// </summary>
        void ValidateBirthDateAndHireDate()
        {
            if ((BirthDate != null) && (!HireDate.IsEmpty()) && (DateTime.Compare((DateTime)BirthDate, HireDate) >= 0))
                throw new InvalidOperationException("Exception!!! BirthDate should be earlier than HireDate!");
        }
        
        //required field
        partial void OnAddressChanging(global::System.String value)
        {
            if (value.IsNull()) throw new InvalidOperationException(nullValueErrorMessage + "Address");
            else if (value.IsEmpty()) throw new InvalidOperationException(emptyValueErrorMessage + "Address");
        }

        //optional field
        partial void OnCityChanging(global::System.String value)
        {
            if (value.IsEmpty()) throw new InvalidOperationException(emptyValueErrorMessage + "City");
        }

        //optional field
        partial void OnRegionChanging(global::System.String value)
        {
            if (value.IsEmpty()) throw new InvalidOperationException(emptyValueErrorMessage + "Region");
        }

        //optional field
        partial void OnPostalCodeChanging(global::System.String value)
        {
            if (value.IsEmpty()) throw new InvalidOperationException(emptyValueErrorMessage + "PostalCode");
        }

        //required field
        partial void OnCountryChanging(global::System.String value)
        {
            if (value.IsNull()) throw new InvalidOperationException(nullValueErrorMessage + "Country");
            else if (value.IsEmpty()) throw new InvalidOperationException(emptyValueErrorMessage + "Country");
        }

        //required field
        partial void OnHomePhoneChanging(global::System.String value)
        {
            if (value.IsNull()) throw new InvalidOperationException(nullValueErrorMessage + "HomePhone");
            else if (value.IsEmpty()) throw new InvalidOperationException(emptyValueErrorMessage + "HomePhone");
        }
        
        //optional field - with default value
        partial void OnExtensionChanged()
        {
            if (Extension.IsEmpty() || Extension.IsNull())
            {
                Extension = "n/a";
            }
        }

        //optional field
        partial void OnPhotoChanging(global::System.Byte[] value)
        {
            if (value.IsEmpty()) throw new InvalidOperationException(emptyValueErrorMessage + "Photo");
        }

        //optional field
        partial void OnNotesChanging(global::System.String value)
        {
            if (value.IsEmpty()) throw new InvalidOperationException(emptyValueErrorMessage + "Notes");
        }

        //optional field
        partial void OnReportsToChanging(global::System.Int32? value)
        {
            if (value.IsEmpty()) throw new InvalidOperationException(emptyValueErrorMessage + "ReportsTo");
        }
 
    }
}
