/****************** Copyright Notice *****************
 
This code is licensed under Microsoft Public License (Ms-PL). 
You are free to use, modify and distribute any portion of this code. 
The only requirement to do that, you need to keep the developer name, as provided below to recognize and encourage original work:

=======================================================
   
Architecture Designed and Implemented By:
Mohammad Ashraful Alam
Microsoft Most Valuable Professional, ASP.NET 2007  2011
Twitter: http://twitter.com/AshrafulAlam | Blog: http://blog.ashraful.net | Portfolio: http://www.ashraful.net
   
*******************************************************/

using System;
using System.Diagnostics;
using System.IO;
using System.Runtime.CompilerServices;
using System.Text;
using System.Xml;

namespace Eisk.Helpers
{

    /// <summary>
    /// Utility class for logging error
    /// Design and Architecture: Mohammad Ashraful Alam [http://www.ashraful.net]
    /// </summary>
    public sealed class Logger
    {
        private Logger() { }

        /// <summary>
        /// Sends mail about the lastly occured error to the admin
        /// </summary>
        public static void ReportLastError()
        {
            System.Exception ex = System.Web.HttpContext.Current.Server.GetLastError();
            //string path = System.Web.HttpContext.Current.Server.MapPath("~/log/log.xml");
            //MyErrorLogger.Utility.ErrorLogWriter x = new MyErrorLogger.Utility.ErrorLogWriter(path);
            //x.WriteLine(ex.ToString());
            //x.Dispose();
            LogToFile(ex);
            
            //Emailer.SendMail("ashraf@mvps.org", "joy_csharp@yahoo.com", logTextHeader, logTextBody, null);

        }

        static void LogToFile(Exception ex)
        {
            string path = System.Web.HttpContext.Current.Server.MapPath("~/system/log/log.txt");
            FileStream fStream = new FileStream(path, FileMode.Append, FileAccess.Write);
            BufferedStream bfs = new BufferedStream(fStream);
            StreamWriter sWriter = new StreamWriter(bfs);
            
            //insert a separator line
            sWriter.WriteLine("=================================================================================================");
            
            //create log for header
            sWriter.WriteLine(ex.Message);
            sWriter.WriteLine("Log time:" + DateTime.Now);
            
            //create log for body
            sWriter.WriteLine(ex.ToString());
            
            //insert a separator line
            sWriter.WriteLine("=================================================================================================");

            sWriter.Close();
            
        }
        /// <summary>
        /// Formates the error message
        /// </summary>
        /// <param name="logText">logged error message</param>
        /// <returns>formatted error message</returns>
        static string FormatLogText(string logText)
        {
            return "[ My Site ]" + " at: " + System.DateTime.Now.ToString() + " says: " + logText;
        }
    }
}


namespace MyErrorLogger.Utility{
    /// <summary>

    /// ErrorLogWriter class

    /// </summary>

    public class ErrorLogWriter:TextWriter{
        // Variables

        private bool Disposed;
        private XmlTextWriter Writer;

        // Constructor

        public ErrorLogWriter(string FilePath){
            Disposed=false;
            Writer=new XmlTextWriter(FilePath,Encoding.Unicode);

            // Write header

            Writer.Formatting=Formatting.Indented;
            Writer.WriteStartDocument();
            Writer.WriteStartElement("error");
            Writer.Flush();
            }

        // Destructor (equivalent to Finalize() without the need to call base.Finalize())

        ~ErrorLogWriter(){
            Dispose(false);
            }

        // Free resources immediately

        protected override void Dispose(bool Disposing){
            if(!Disposed){
                if(Disposing){
                    }
                // Close file

                Writer.Close();
                Writer=null;
                // Disposed

                Disposed=true;
                // Parent disposing

                base.Dispose(Disposing);
                }
            }

        // Close the file

        public override void Close(){
            // Write footer

            Writer.WriteEndElement();
            Writer.WriteEndDocument();
            Writer.Flush();
            // Free resources

            Dispose(true);
            GC.SuppressFinalize(this);
            }

        // Implement Encoding() method from TextWriter

        public override Encoding Encoding{
            get{
                return(Encoding.Unicode);
                }
            }

        // Implement WriteLine() method from TextWriter (remove MethodImpl attribute for single-threaded app)

        // Use stack trace and reflection to get the calling class and method

        [MethodImpl(MethodImplOptions.Synchronized)]
        public override void WriteLine(string Txt){
            Writer.WriteStartElement("event");
            Writer.WriteStartElement("time");
            Writer.WriteString(DateTime.Now.ToLongTimeString());
            Writer.WriteEndElement();
            Writer.WriteStartElement("class");
            Writer.WriteString(new StackTrace().GetFrame(2).GetMethod().ReflectedType.FullName);
            Writer.WriteEndElement();
            Writer.WriteStartElement("method");
            Writer.WriteString(new StackTrace().GetFrame(2).GetMethod().ToString());
            Writer.WriteEndElement();
            Writer.WriteStartElement("text");
            Writer.WriteString(Txt);
            Writer.WriteEndElement();
            Writer.WriteEndElement();
            Writer.Flush();
            }
        }
    }

